extern crate ctap_hmac as ctap;
use ctap::{FidoCredentialRequestBuilder, FidoDevice};
use hex;
use std::env::args;
use std::io::stdout;
use std::io::Write;
use structopt;
use structopt::StructOpt;

#[derive(Debug, StructOpt)]
#[structopt(name = "fido2pam")]
struct CliOpt {
    /// Whether the authenticator should promt the user for verification
    #[structopt(short, long = "user-verification")]
    uv: bool,

    /// Username for which the credential will be requested
    username: String,
}

fn main() -> Result<(), ctap::FidoError> {
    let opt = CliOpt::from_args();
    let device_info = ctap::get_devices()?.next().expect("no device");
    let mut device = ctap::FidoDevice::new(&device_info)?;

    let req = FidoCredentialRequestBuilder::default()
        .rp_id("fido2pam")
        .user_name(opt.username.as_ref())
        .uv(opt.uv)
        .build()
        .unwrap();

    let cred = device.make_credential(&req)?;

    stdout()
        .write_all(
            &[
                &opt.username,
                &hex::encode(&cred.id)[..],
                &hex::encode(&cred.public_key.unwrap())[..],
            ]
            .join(":")
            .as_bytes(),
        )
        .unwrap();
    Ok(())
}
